@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014, Robin Hahling
+ * Copyright (c) 2012-2017, Robin Hahling
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,9 +40,6 @@
 #include "list.h"
 #include "extern.h"
 
-char *g_unknown_str = "unknown";
-char *g_none_str    = "none";
-
 /*
  * Initializes a queue structure
  * @lst: queue pointer
@@ -52,10 +49,6 @@ init_queue(struct list *lst)
 {
 	lst->head  = NULL;
 	lst->tail   = NULL;
-	lst->fsmaxlen = -1;
-	lst->dirmaxlen = -1;
-	lst->typemaxlen = -1;
-	lst->mntoptmaxlen = -1; /* stairway to heaven ! */
 }
 
 /*
@@ -70,10 +63,8 @@ is_empty(struct list lst)
 {
 	if (lst.head == NULL)
 		return 1;
-		/* NOTREACHED */
 	else
 		return 0;
-		/* NOTREACHED */
 }
 
 /*
@@ -82,7 +73,7 @@ is_empty(struct list lst)
  * @elt: element
  * Returns:
  *	--> -1 on error
- *	-->  0 on sucess
+ *	-->  0 on success
  */
 int
 enqueue(struct list *lst, struct fsmntinfo fmi)
@@ -92,7 +83,6 @@ enqueue(struct list *lst, struct fsmntinfo fmi)
 	if (new_fmi == NULL) {
 		(void)fputs("Error while allocating memory to fmi", stderr);
 		return -1;
-		/* NOTREACHED */
 	}
 
 	/* initialize the new element to be inserted in the queue */
@@ -106,7 +96,6 @@ enqueue(struct list *lst, struct fsmntinfo fmi)
 	lst->tail = new_fmi;
 
 	return 0;
-	/* NOTREACHED */
 }
 
 /*
@@ -119,11 +108,17 @@ fmi_init(void)
 {
 	struct fsmntinfo fmi;
 
-	fmi.fsname = g_unknown_str;
-	fmi.dir    = g_unknown_str;
-	fmi.type   = g_unknown_str;
-	fmi.opts   = g_none_str;
+	fmi.fsname  = g_unknown_str;
+	fmi.fstype  = g_unknown_str;
+	fmi.mntdir  = g_unknown_str;
+	fmi.mntopts = g_none_str;
 
+	fmi.perctused = 0.0;
+	fmi.total     = 0.0;
+	fmi.avail     = 0.0;
+	fmi.used      = 0.0;
+
+	fmi.flags  = 0;
 	fmi.bsize  = 0;
 	fmi.frsize = 0;
 	fmi.blocks = 0;
@@ -136,7 +131,6 @@ fmi_init(void)
 	fmi.next   = NULL;
 
 	return fmi;
-	/* NOTREACHED */
 }
 
 struct fsmntinfo
@@ -150,12 +144,12 @@ struct fsmntinfo
 
 	if(p->fsname != g_unknown_str) /* we malloc'd a string */
 		free(p->fsname);
-	if(p->dir != g_unknown_str)
-		free(p->dir);
-	if(p->type != g_unknown_str)
-		free(p->type);
-	if(p->opts != g_none_str)
-		free(p->opts);
+	if(p->fstype != g_unknown_str)
+		free(p->fstype);
+	if(p->mntdir != g_unknown_str)
+		free(p->mntdir);
+	if(p->mntopts != g_none_str)
+		free(p->mntopts);
 
 	free(p);
 
