@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014, Robin Hahling
+ * Copyright (c) 2012-2017, Robin Hahling
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,29 +39,24 @@
 
 #include <sys/types.h>
 
-#if defined(__APPLE__)
-#include <sys/mount.h>
-#include <sys/param.h>
-#endif /* __APPLE__ */
-
-
 /*
- * structure needed to store informations about mounted fs
- * It should contain brut datas.
- * Later on, we would need to compute those infos:
- * unsigned long bused;	used blocks
- * double prct_used;	usage of fs in percent
- * double prct_free;	free space of fs in percent
+ * Structure to store information about mounted fs
  */
 struct fsmntinfo {
 	/* infos to get from getmntent(3) */
 	char *fsname;	/* name of mounted file system */
-	char *dir;	/* file system path prefix */
-	char *type;	/* mount type */
-	char *opts;	/* mount options (see mntent.h) */
+	char *fstype;	/* mount type */
+	char *mntdir;	/* file system path prefix */
+	char *mntopts;	/* mount options (see mntent.h) */
+
+	double perctused;   /* fs usage in % */
+	double total;	    /* fs total size */
+	double avail;	    /* fs available size */
+	double used;	    /* fs used size */
 
 	/* infos to get from statvfs(3) */
-#if defined(__linux__) || defined(__NetBSD__)
+#if defined(__linux__) || defined(__GLIBC__)
+	int		flags;	/* XXX: does not exist on Linux */
 	unsigned long	bsize;	/* file system block size */
 	unsigned long	frsize;	/* fragment size */
 	fsblkcnt_t	blocks;	/* size of fs in frsize unit */
@@ -70,18 +65,31 @@ struct fsmntinfo {
 	fsfilcnt_t	files;	/* # of inodes */
 	fsfilcnt_t	ffree;	/* # of free inodes */
 	fsfilcnt_t	favail;	/* # of available inodes */
-#endif /* __linux__ || __NetBSD__ */
+#endif /* __linux__ */
+#if defined(__NetBSD__)
+	unsigned long	flags;	/* mount exported flags */
+	unsigned long	bsize;	/* file system block size */
+	unsigned long	frsize;	/* fragment size */
+	fsblkcnt_t	blocks;	/* size of fs in frsize unit */
+	fsblkcnt_t	bfree;	/* # of free blocks */
+	fsblkcnt_t	bavail;	/* # of available blocks */
+	fsfilcnt_t	files;	/* # of inodes */
+	fsfilcnt_t	ffree;	/* # of free inodes */
+	fsfilcnt_t	favail;	/* # of available inodes */
+#endif /* __NetBSD__ */
 #if defined(__FreeBSD__)
+	uint64_t	flags;	/* mount exported flags */
 	uint64_t	bsize;	/* file system block size */
 	unsigned long   frsize;	/* XXX: does not exist on FreeBSD */
 	uint64_t	blocks;	/* size of fs in frsize unit */
 	uint64_t	bfree;	/* # of free blocks */
-	uint64_t	bavail;	/* # of available blocks */
+	int64_t		bavail;	/* # of available blocks */
 	uint64_t	files;	/* # of inodes */
 	int64_t		ffree;	/* # of free nodes to non super-user */
 	unsigned long	favail;	/* XXX: does not exist on FreeBSD */
 #endif /* __FreeBSD__ */
 #if defined(__OpenBSD__)
+	u_int32_t	flags;	/* mount exported flags */
 	u_int32_t	bsize;	/* file system block size */
 	unsigned long	frsize;	/* XXX: does not exist on OpenBSD */
 	u_int64_t	blocks;	/* size of fs in frsize unit */
@@ -92,16 +100,18 @@ struct fsmntinfo {
 	int64_t		favail;	/* # of available inodes */
 #endif /* __OpenBSD__ */
 #if defined(__DragonFly__)
-	long bsize;		/* file system block size */
-	unsigned long frsize;	/* XXX: does not exist on DragonFly */
-	long blocks;		/* size of fs in frsize unit */
-	long bfree;		/* # of free blocks */
-	long bavail;		/* # of available blocks */
-	long files;		/* # of inodes */
-	long ffree;		/* # of free inodes */
-	unsigned long favail;	/* XXX: does not exist on DragonFly */
+	int		flags;	/* mount exported flags */
+	long		bsize;	/* file system block size */
+	unsigned long	frsize;	/* XXX: does not exist on DragonFly */
+	long		blocks;	/* size of fs in frsize unit */
+	long		bfree;	/* # of free blocks */
+	long		bavail;	/* # of available blocks */
+	long		files;	/* # of inodes */
+	long		ffree;	/* # of free inodes */
+	unsigned long	favail;	/* XXX: does not exist on DragonFly */
 #endif /* __DragonFly__ */
 #if defined(__APPLE__)
+	uint32_t	flags;	/* mount exported flags */
 	uint32_t	bsize;	/* file system block size */
 	unsigned long	frsize;	/* XXX: does not exist on OSX */
 	uint64_t	blocks;	/* size of fs in frsize unit */
@@ -111,7 +121,19 @@ struct fsmntinfo {
 	uint64_t	ffree;	/* # of free inodes */
 	unsigned long	favail;	/* XXX: does not exist on OSX */
 #endif /* __APPLE__ */
+#if defined(__sun)
+	int		flags;	/* XXX: does not exist on Solaris */
+	u_long		bsize;	/* file system block size */
+	u_long		frsize;	/* fragment size */
+	fsblkcnt_t	blocks;	/* size of fs in frsize unit */
+	fsblkcnt_t	bfree;	/* # of free blocks */
+	fsblkcnt_t	bavail;	/* # of available blocks */
+	fsfilcnt_t	files;	/* # of inodes */
+	fsfilcnt_t	ffree;	/* # of free inodes */
+	fsfilcnt_t	favail;	/* # of available inodes */
+#endif /* __sun */
 
+	int ignored;
 	/* pointer to the next element of the list */
 	struct fsmntinfo *next;
 };
@@ -120,11 +142,6 @@ struct fsmntinfo {
 struct list {
 	struct fsmntinfo *head;
 	struct fsmntinfo *tail;
-
-	int fsmaxlen; /* should be the size of the longest fsname */
-	int dirmaxlen; /* same for dir */
-	int typemaxlen; /* same for type */
-	int mntoptmaxlen; /* same for mount option */
 };
 
 /* function declaration */
