@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014, Robin Hahling
+ * Copyright (c) 2012-2017, Robin Hahling
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,6 @@
  *
  * Various util functions
  */
-#define _BSD_SOURCE
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -44,7 +43,11 @@
 #include <ctype.h>
 #include <time.h>
 #include <sys/ioctl.h>
-#include <inttypes.h>
+#include <math.h>
+
+#if defined(__sun)
+#include <termios.h> /* on solaris, this is where struct winsize is declared */
+#endif /* __sun */
 
 #include "util.h"
 
@@ -52,6 +55,25 @@
 #include <libintl.h>
 #endif
 
+/* static functions declaration */
+static int typecmp(const void *e1, const void *e2);
+
+/*
+ * Return the number of digits in `n`
+ * @n: a positive integer.
+ */
+static int
+count_digit(unsigned long long n)
+{
+	int i = 0;
+
+	do {
+		++i;
+		n /= 10;
+	} while (n != 0);
+	return i;
+}
+
 /*
  * Return the longest of the two parameters
  * @a: first element to compare
@@ -61,7 +83,6 @@ int
 imax(int a, int b)
 {
 	return (a > b ? a : b);
-	/* NOTREACHED */
 }
 
 /*
@@ -75,14 +96,12 @@ strtrim(char *str)
 
 	if (!str)
 		return NULL;
-		/* NOTREACHED */
 
 	while (isspace(*str))
 		str++;
 
 	if (*str == '\0')
 		return str;
-		/* NOTREACHED */
 
 	end = str + strlen(str) - 1;
 	while (end > str && isspace(*end))
@@ -91,7 +110,6 @@ strtrim(char *str)
 	*(end + 1) = '\0';
 
 	return str;
-	/* NOTREACHED */
 }
 
 /*
@@ -106,14 +124,12 @@ shortenstr(char *str, int len)
 
 	if (slen < len + 1)
 		return str;
-		/* NOTREACHED */
 
 	str += (slen - len);
 
 	str[0] = '+';
 
 	return str;
-	/* NOTREACHED */
 }
 
 /*
@@ -138,13 +154,9 @@ sanitizestr(const char *str)
 
 	if (i == (int)nchars) /* No '_' was found. */
 		return strdup(str);
-		/* NOTREACHED */
 
-	if ((new = malloc(nchars)) == NULL) {
-		(void)fputs("malloc failed\n", stderr);
+	if ((new = malloc(nchars)) == NULL)
 		return NULL;
-		/* NOTREACHED */
-	}
 
 	for (i = 0; str[i] != '\0'; i++) {
 		if (str[i] == '_') {
@@ -160,44 +172,6 @@ sanitizestr(const char *str)
 	new[nchars-1] = '\0';
 
 	return new;
-	/* NOTREACHED */
-}
-
-/*
- * Get the width of tty and return it.
- * Return 0 if stdout is not a tty.
- */
-int
-getttywidth(void)
-{
-	int width = 0;
-#ifdef TIOCGSIZE
-	struct ttysize win;
-#elif defined(TIOCGWINSZ)
-	struct winsize win;
-#endif /* TIOCGSIZE */
-
-	if (!isatty(STDOUT_FILENO))
-		return 0;
-		/* NOTREACHED */
-
-#ifdef TIOCGSIZE
-	if (ioctl(STDOUT_FILENO, TIOCGSIZE, &win) == 0)
-#if defined(__APPLE__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
-		width = win.ts_cols;
-#else
-		width = win.ws_col;
-#endif /* __APPLE__ || __NetBSD__ || __OpenBSD__ || __DragonFly__ */
-#elif defined(TIOCGWINSZ)
-	if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &win) == 0)
-#if defined(__APPLE__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
-		width = win.ts_cols;
-#else
-		width = win.ws_col;
-#endif /* __APPLE__ || __NetBSD__ || __OpenBSD__ || __DragonFly__ */
-#endif /* TIOCGSIZE */
-	return width == 0 ? 80 : width;
-	/* NOTREACHED */
 }
 
 /*
@@ -221,7 +195,6 @@ humanize(double *n)
 	}
 
     return i;
-    /* NOTREACHED */
 }
 
 /*
@@ -240,7 +213,6 @@ humanize_i(uint64_t *n)
 	}
 
     return i;
-    /* NOTREACHED */
 }
 
 /*
@@ -262,39 +234,30 @@ print_unit(int i, int mode)
 			else
 				(void)printf(" ");
 			return;
-			/* NOTREACHED */
 		case 1: /* Kio  or Ko */
 			(void)printf("K");
 			return;
-			/* NOTREACHED */
 		case 2: /* Mio or Mo */
 			(void)printf("M");
 			return;
-			/* NOTREACHED */
 		case 3: /* Gio or Go*/
 			(void)printf("G");
 			return;
-			/* NOTREACHED */
 		case 4: /* Tio or To*/
 			(void)printf("T");
 			return;
-			/* NOTREACHED */
 		case 5: /* Pio or Po*/
 			(void)printf("P");
 			return;
-			/* NOTREACHED */
 		case 6: /* Eio or Eo*/
 			   (void)printf("E");
 			return;
-			/* NOTREACHED */
 		case 7: /* Zio or Zo*/
 			(void)printf("Z");
 			return;
-			/* NOTREACHED */
 		case 8: /* Yio or Yo*/
 			(void)printf("Y");
 			return;
-			/* NOTREACHED */
 		default:
 			(void)fputs("Could not print unit type in"
 					" human-readable format\n", stderr);
@@ -305,39 +268,30 @@ print_unit(int i, int mode)
 		else
 			(void)printf(" ");
 		return;
-		/* NOTREACHED */
 	case 'k':
 		(void)printf("K");
 		return;
-		/* NOTREACHED */
 	case 'm':
 		(void)printf("M");
 		return;
-		/* NOTREACHED */
 	case 'g':
 		(void)printf("G");
 		return;
-		/* NOTREACHED */
 	case 't':
 		(void)printf("T");
 		return;
-		/* NOTREACHED */
 	case 'p':
 		(void)printf("P");
 		return;
-		/* NOTREACHED */
 	case 'e':
 		(void)printf("E");
 		return;
-		/* NOTREACHED */
 	case 'z':
 		(void)printf("Z");
 		return;
-		/* NOTREACHED */
 	case 'y':
 		(void)printf("Y");
 		return;
-		/* NOTREACHED */
 	default:
 		(void)fputs("Could not print unit type\n", stderr);
 	}
@@ -356,181 +310,86 @@ cvrt(double n)
 	switch (unitflag) {
 	case 'b':
 		return n;
-		/* NOTREACHED */
 	case 'e':
 		if (mflag) /* 1000^6 */
 			return n / 1000000000000000000.0;
-			/* NOTREACHED */
 		else /* 1024^6 */
 			return n / 1152921504606846976.0;
-			/* NOTREACHED */
 	case 'g':
 		if (mflag) /* 1000^3 */
 			return n / 1000000000.0;
-			/* NOTREACHED */
 		else /* 1024^3 */
 			return n / 1073741824.0;
-			/* NOTREACHED */
 	case 'k':
 		if (mflag)
 			return n / 1000.0;
-			/* NOTREACHED */
 		else
 			return n / 1024.0;
-			/* NOTREACHED */
 	case 'm':
 		if (mflag) /* 1000^2 */
 			return n / 1000000.0;
-			/* NOTREACHED */
 		else /* 1024^2 */
 			return n / 1048576.0;
-			/* NOTREACHED */
 	case 'p':
 		if (mflag) /* 1000^5 */
 			return n / 1000000000000000.0;
-			/* NOTREACHED */
 		else /* 1024^5 */
 			return n / 1125899906842624.0;
-			/* NOTREACHED */
 	case 't':
 		if (mflag) /* 1000^4 */
 			return n / 1000000000000.0;
-			/* NOTREACHED */
 		else /* 1024^4 */
 			return n / 1099511627776.0;
-			/* NOTREACHED */
 	case 'y':
 		if (mflag) /* 1000^8 */
 			return n / 1000000000000000000000000.0;
-			/* NOTREACHED */
 		else /* 1024^8 */
 			return n / 1208925819614629174706176.0;
-			/* NOTREACHED */
 	case 'z':
 		if (mflag)
 			/* 1000^7 */
 			return n / 1000000000000000000000.0;
-			/* NOTREACHED */
 		else /* 1024^7 */
 			return n / 1180591620717411303424.0;
-			/* NOTREACHED */
 	default:
 		(void)fputs("Could not convert unit size\n", stderr);
 		return n;
+	}
+}
+
+/*
+ * Return:
+ *	1 if the given fs should be showed
+ *	0 if the given fs should be skipped
+ * @fs: fs type or name to check
+ * @filter: filter string
+ * @nm: boolean indicating if the negative matching is activated
+ */
+int
+fsfilter(const char *fs, const char *filter, int nm)
+{
+	int ret = 0; /* assume it should not be shown */
+	char *stropt;
+	char *strtmp;
+
+	if ((strtmp = strdup(filter)) == NULL) {
+		(void)fputs("Cannot duplicate filter\n", stderr);
+		exit(EXIT_FAILURE);
 		/* NOTREACHED */
 	}
-}
 
-/*
- * Return:
- *	1 if the given fs should be showed
- *	0 if the given fs should be skipped
- * @type: fs type to check
- * @filter: filter string
- * @nm: boolean indicating if the negative matching is activated
- */
-int
-fstypefilter(char *type, char *filter, int nm)
-{
-	int ret = 1;
-	char *stropt;
-	char *strtmp;
-
-	if (tflag) {
-		if ((strtmp = strdup(filter)) == NULL) {
-			(void)fputs("Cannot duplicate filter\n", stderr);
-			exit(EXIT_FAILURE);
-			/* NOTREACHED */
+	stropt = strtok(strtmp, ",");
+	while (stropt != NULL) {
+		if (strncmp(fs, stropt, strlen(stropt)) == 0) {
+			ret = 1;
+			break;
 		}
-		/* assume it should not be shown */
-		ret = 0;
-		stropt = strtok(strtmp, ",");
-		while (stropt != NULL) {
-			if (strcmp(type, stropt) == 0) {
-				ret = 1;
-				break;
-			}
-			stropt = strtok(NULL, ",");
-		}
-		free(strtmp);
+		stropt = strtok(NULL, ",");
 	}
+	free(strtmp);
 
 	/* reverse result if negative matching activated */
-	if (nm) {
-		if (ret)
-			ret = 0;
-		else
-			ret = 1;
-	}
-
-	return ret;
-	/* NOTREACHED */
-}
-
-/*
- * Return:
- *	1 if the given fs should be showed
- *	0 if the given fs should be skipped
- * @type: fs type to check
- * @filter: filter string
- * @nm: boolean indicating if the negative matching is activated
- */
-int
-fsnamefilter(char *fsname, char *filter, int nm)
-{
-	int ret = 1;
-	char *stropt;
-	char *strtmp;
-
-	if (pflag) {
-		if ((strtmp = strdup(filter)) == NULL) {
-			(void)fputs("Cannot duplicate filter\n", stderr);
-			exit(EXIT_FAILURE);
-			/* NOTREACHED */
-		}
-		/* assume it should not be shown */
-		ret = 0;
-		stropt = strtok(strtmp, ",");
-		while (stropt != NULL) {
-			if (strncmp(fsname, stropt, strlen(stropt)) == 0) {
-				ret = 1;
-				break;
-			}
-			stropt = strtok(NULL, ",");
-		}
-		free(strtmp);
-	}
-
-	/* reverse result if negative matching activated */
-	if (nm) {
-		if (ret)
-			ret = 0;
-		else
-			ret = 1;
-	}
-
-	return ret;
-	/* NOTREACHED */
-}
-
-/*
- * Determine if the fstype is remote or only local.
- * Return 1 if remote, otherwise return 0.
- * @fstype: type of file system
- */
-int
-is_remote(char *fstype)
-{
-    const char remote_fs[] = "afs cifs coda fuse.sshfs mfs "
-                             "ncpfs ftpfs nfs smbfs sshfs";
-	/* assume it is local by default */
-	int ret = 0;
-
-	if (strstr(remote_fs, fstype))
-		ret = 1;
-
-	return ret;
-	/* NOTREACHED */
+	return nm ? !ret : ret;
 }
 
 /*
@@ -544,22 +403,18 @@ cmp(struct fsmntinfo *a, struct fsmntinfo *b)
 	switch(qflag) {
 	case 1:
 		return strcmp(a->fsname, b->fsname);
-		/* NOTREACHED */
 	case 2:
-		return strcmp(a->type, b->type);
-		/* NOTREACHED */
+		return strcmp(a->fstype, b->fstype);
 	case 3:
-		return strcmp(a->dir, b->dir);
-		/* NOTREACHED */
+		return strcmp(a->mntdir, b->mntdir);
 	default:
 		return -1;
-		/* NOTREACHED */
 	}
 }
 
 /*
  * Perform a mergesort algorithm to sort the list by ascending
- * Results depends on what was chosen for comparison (fsname, type or dir)
+ * Results depends on what was chosen for comparison (fsname, fstype or mntdir)
  * @fmi: pointer to the first element of the linked list structure to be sorted
  */
 struct fsmntinfo *
@@ -575,7 +430,6 @@ msort(struct fsmntinfo *fmi)
 	 */
 	if (fmi == NULL || fmi->next == NULL)
 		return fmi;
-		/* NOTREACHED */
 
 	do {
 		nmerges = 0;
@@ -621,156 +475,196 @@ msort(struct fsmntinfo *fmi)
 	} while (nmerges > 1);
 
 	return fmi;
-	/* NOTREACHED */
 }
 
 /*
- * auto-adjust options based on the size needed to display the informations
- * @lst: list containing info
- * @width: width of the output
+ * Get the with of TTY and retun it.
+ * 0 is returned if stdout is not a tty.
+ */
+int
+getttywidth(void)
+{
+	int width = 0;
+	struct winsize win;
+
+	if (!isatty(STDOUT_FILENO))
+		return 0;
+
+	if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &win) == 0)
+		width = win.ws_col;
+
+	return width == 0 ? 80 : width;
+}
+
+/*
+ * init a maxwidths structure
  */
 void
-auto_adjust(struct list lst, int width)
+init_maxwidths(void)
 {
-	int req, gap;
+	/*
+	 * init min width to header names and width of the graph bar + 1 to have
+	 * a space between each column.
+	 */
+	max.fsname	= (int)strlen(_("FILESYSTEM")) + 1;
+	max.fstype	= Tflag ? (int)strlen(_("TYPE")) + 1 : 0;
+	max.bar		= bflag ? 0 : wflag ? GRAPHBAR_WIDE : GRAPHBAR_SHORT;
+	max.perctused	= (int)strlen(_("%USED")) + 1;
+	max.used	= dflag ? (int)strlen(_("USED")) + 1 : 0;
+	max.avail	= (int)strlen(_("AVAILABLE")) + 1;
+	max.total	= (int)strlen(_("TOTAL")) + 1;
+	max.nbinodes	= iflag ? (int)strlen(_("#INODES")) + 1 : 0;
+	max.avinodes	= iflag ? (int)strlen(_("AV.INODES")) + 1 : 0;
+	max.mntdir	= Mflag ? 0 : (int)strlen(_("MOUNTED ON")) + 1;
+	max.mntopts	= oflag ? (int)strlen(_("MOUNT OPTIONS")) + 1: 0;
+}
 
-	req = req_width(lst);
+/*
+ * Return the required width necessary for the number to be displayed.
+ * This functions takes into account the unit used (b, k, m, g, ...).
+ * @fs_size: file system size from which the required width for displaying
+ * should be computed.
+ */
+int
+get_req_width(double fs_size)
+{
+	long i, index;
+	int req_width, req_min;
+	const char *unitstring = "bkmgtpezy";
+	char *match;
 
-	/* nothing to adjust here */
-	if ((gap = (width - req)) >= 0)
+	/* spaces for the unit symbol and floating point */
+	req_min = 4;
+	req_width = req_min;
+
+	if (unitflag == 'h') {
+		req_width += 3;
+	} else {
+		if ((match = strchr(unitstring, unitflag)) == NULL) {
+			(void)fputs(_("Cannot compute required width\n"),
+			stderr);
+			return -1;
+		}
+
+		if ((fs_size > 0.0) && isnormal(fs_size)) {
+			/*
+			 * casting from double to int shall be safe here since
+			 * log10 of even the largest possible double value (ie:
+			 * around 309 in most cases) shall fit into an integer
+			 */
+			req_width += (int)(1.0 + trunc(log10(fs_size)));
+		}
+
+		index = match - unitstring + 1;
+		for (i = 1; i < index; i++) {
+			/*
+			 * displaying the same number in a "greater" unit (cf
+			 * unitstring) requires 3 digits less than what was
+			 * previously required
+			 */
+			req_width -= 3;
+		}
+	}
+
+	return (req_width < req_min) ? req_min : req_width;
+}
+
+/*
+ * Update values of the maxwidth structure.
+ * @fmi: structure containing filesystem information
+ */
+void
+update_maxwidth(struct fsmntinfo *fmi)
+{
+	if (!aflag && (is_mnt_ignore(fmi) == 1))
 		return;
-		/* NOTREACHED */
+
+	/* + 1 for a space between each column */
+	max.fsname = imax((int)strlen(fmi->fsname) + 1, max.fsname);
+	max.fstype = imax((int)strlen(fmi->fstype) + 1, max.fstype);
+
+	if (!Mflag)
+		max.mntdir = imax((int)strlen(fmi->mntdir) + 1, max.mntdir);
+
+	if (oflag)
+		max.mntopts = imax((int)strlen(fmi->mntopts) + 1, max.mntopts);
+
+	if (dflag)
+		max.used = imax(get_req_width(fmi->used), max.used);
+	max.avail = imax(get_req_width(fmi->avail), max.avail);
+	max.total = imax(get_req_width(fmi->total), max.total);
+
+	if (iflag) {
+		max.nbinodes = imax(2 + count_digit(fmi->files), max.nbinodes);
+		max.avinodes = imax(3 + count_digit(fmi->ffree), max.avinodes);
+	}
+}
+
+/*
+ * auto-adjust options based on the size needed to display the information
+ * @lst: list containing info
+ * @tty_width: width of the output terminal
+ */
+void
+auto_adjust(int tty_width)
+{
+	int req_width;
+
+	req_width = max.fsname + max.fstype + max.bar + max.perctused + max.used
+		    + max.avail + max.total + max.nbinodes + max.avinodes
+		    + max.mntdir + max.mntopts;
+
+	if (tty_width > req_width)
+		return; /* nothing to adjust */
 
 	(void)fputs(_("WARNING: TTY too narrow. Some options have been disabled"
 		" to make dfc output fit (use -f to override).\n"), stderr);
-
 	if (!bflag) {
-		/* large graph should be the first option to disable */
 		if (wflag) {
 			wflag = 0;
-			gap += 30;
-			if (gap >= 0)
+			req_width -= GRAPHBAR_WIDE - GRAPHBAR_SHORT;
+			if (tty_width >= req_width)
 				return;
-				/* NOTREACHED */
 		}
 		bflag = 1;
-		gap += 23;
-		if (gap >= 0)
+		req_width -= GRAPHBAR_SHORT;
+		if (tty_width >= req_width)
 			return;
-			/* NOTREACHED */
 	}
 	if (dflag) {
 		dflag = 0;
-		gap += 4;
-		if (unitflag == 'k')
-			gap += 7;
-		else if (unitflag == 'b')
-			gap += 12;
-		else
-			gap += 6;
-		if (gap >= 0)
+		req_width -= max.used;
+		if (tty_width >= req_width)
 			return;
-			/* NOTREACHED */
 	}
 	if (Tflag) {
 		Tflag = 0;
-		gap += imax(lst.typemaxlen, 5);
-		if (gap >= 0)
+		req_width -= max.fstype;
+		if (tty_width >= req_width)
+			return;
+	}
+	if (!Mflag) {
+		Mflag = 1;
+		req_width -= max.mntdir;
+		if (tty_width >= req_width)
 			return;
-			/* NOTREACHED */
 	}
 	if (iflag) {
 		iflag = 0;
-		gap += 20;
-		if (gap >= 0)
+		req_width -= max.nbinodes;
+		req_width -= max.avinodes;
+		if (tty_width >= req_width)
 			return;
-			/* NOTREACHED */
 	}
 	if (oflag) {
 		oflag = 0;
-		gap += imax(lst.mntoptmaxlen, 13);
-		if (gap >= 0)
+		req_width -= max.mntopts;
+		if (tty_width >= req_width)
 			return;
-			/* NOTREACHED */
-	}
-	if (gap < 0)
-		(void)fputs(_("WARNING: Output still messed up. Enlarge your "
-				"terminal if you can...\n"), stderr);
-}
-
-/*
- * compute the required width needed for the output and return it
- * (computation based on text_disp_header function)
- * @lst: list containing the info
- */
-int
-req_width(struct list lst)
-{
-	int ret;
-
-	/* dir and fs are always displayed */
-	ret = imax(lst.fsmaxlen, 11);
-
-	if (Tflag)
-		ret += imax(lst.typemaxlen, 5);
-	if (!bflag) {
-		ret += 23;
-		if (wflag)
-			ret += 30;
-	}
-	/* % */
-	ret += 5;
-
-	if (dflag) {
-		if (unitflag == 'k')
-			ret += 7;
-		else if (unitflag == 'b')
-			ret += 12;
-		else
-			ret += 6;
-		ret += 4;
 	}
 
-	/* available */
-	ret += 9;
-
-	switch (unitflag) {
-	case 'b':
-		ret += 7 + 11;
-		break;
-	case 'k':
-		ret += 2 + 6;
-		break;
-	case 'm':
-		ret += 5;
-		break;
-	case 'h': /* FALLTHROUGH */
-	case 'g': /* FALLTHROUGH */
-	case 't': /* FALLTHROUGH */
-	case 'p': /* FALLTHROUGH */
-	case 'e': /* FALLTHROUGH */
-	case 'z': /* FALLTHROUGH */
-	case 'y': /* FALLTHROUGH */
-		ret += 1 + 5;
-		break;
-	default:
-		(void)fputs("Unknown unit type\n", stderr);
-	}
-
-	/* total */
-	ret += 5;
-
-	if (iflag)
-		ret += 20;
-
-	/* mounted on */
-	ret += imax(lst.dirmaxlen, 12);
-
-	if (oflag)
-		ret += imax(lst.mntoptmaxlen, 13);
-
-	return ret;
-	/* NOTREACHED */
+	(void)fputs(_("WARNING: Output still messed up. Enlarge your "
+			"terminal if you can...\n"), stderr);
 }
 
 /*
@@ -805,7 +699,7 @@ fetchdate(void)
  * Return the name of the given color or NULL
  * @color: color as defined in extern.h
  */
-char *
+const char *
 colortostr(int color)
 {
 	switch (color) {
@@ -835,35 +729,26 @@ colortostr(int color)
  * @col: color name
  */
 int
-colortoint(char *col)
+colortoint(const char *col)
 {
-	if (strcoll(col, _("black")) == 0)
+	if (strcoll(col, "black") == 0)
 		return BLACK;
-		/* NOTREACHED */
-	else if (strcoll(col, _("red")) == 0)
+	else if (strcoll(col, "red") == 0)
 		return RED;
-		/* NOTREACHED */
-	else if (strcoll(col, _("green")) == 0)
+	else if (strcoll(col, "green") == 0)
 		return GREEN;
-		/* NOTREACHED */
-	else if (strcoll(col, _("yellow")) == 0)
+	else if (strcoll(col, "yellow") == 0)
 		return YELLOW;
-		/* NOTREACHED */
-	else if (strcoll(col, _("blue")) == 0)
+	else if (strcoll(col, "blue") == 0)
 		return BLUE;
-		/* NOTREACHED */
-	else if (strcoll(col, _("magenta")) == 0)
+	else if (strcoll(col, "magenta") == 0)
 		return MAGENTA;
-		/* NOTREACHED */
-	else if (strcoll(col, _("cyan")) == 0)
+	else if (strcoll(col, "cyan") == 0)
 		return CYAN;
-		/* NOTREACHED */
-	else if (strcoll(col, _("white")) == 0)
+	else if (strcoll(col, "white") == 0)
 		return WHITE;
-		/* NOTREACHED */
 	else
 		return -1;
-		/* NOTREACHED */
 }
 
 /*
@@ -871,22 +756,114 @@ colortoint(char *col)
  * ie: it should be an hexadecimal value
  * Returns 0 if all went well, otherwise it returns -1
  * @color: input color
- * NOTE: color should ommit the #: white is FFFFFF and not #FFFFFF
+ * NOTE: color should omit the #: white is FFFFFF and not #FFFFFF
  */
 int
-chk_html_colorcode(char *color)
+chk_html_colorcode(const char *color)
 {
 	int i;
 
 	if (strlen(color) != HTMLCOLORCODELENGTH)
 		return -1;
-		/* NOTREACHED */
 
 	for (i = 0; i < HTMLCOLORCODELENGTH; i++)
 		if (isxdigit(color[i]) == 0)
 			return -1;
-			/* NOTREACHED */
 
 	return 0;
-	/* NOTREACHED */
+}
+
+
+/*
+ * Comparison function needed for is_pseudofs and is_remotefs bsearch call.
+ */
+static int
+typecmp(const void *e1, const void *e2)
+{
+	const char *s1 = *(const char * const *)e1;
+	const char *s2 = *(const char * const *)e2;
+	return strcmp(s1, s2);
+}
+
+/*
+ * Determine if fsname is a pseudo filesystem or not.
+ * This function is useless under *BSD and OSX systems.
+ * Return 1 if it is, 0 otherwise.
+ */
+int
+is_pseudofs(const char *type)
+{
+	/* keep sorted for binary search */
+	static const char *pseudofs[] = {
+		"anon_inodefs",
+		"autofs",
+		"bdev",
+		"binfmt_misc",
+		"cgroup",
+		"configfs",
+		"cpuset",
+		"debugfs",
+		"devfs",
+		"devpts",
+		"devtmpfs",
+		"dlmfs",
+		"fdescfs",
+		"fuse.gvfs-fuse-daemon",
+		"fusectl",
+		"hugetlbfs",
+		"linprocfs",
+		"mqueue",
+		"nfsd",
+		"none",
+		"pipefs",
+		"proc",
+		"pstore",
+		"ramfs",
+		"rootfs",
+		"rpc_pipefs",
+		"securityfs",
+		"sockfs",
+		"spufs",
+		"sysfs",
+		"tmpfs"
+	};
+
+	if (!type)
+		return 0;
+
+	if (bsearch(&type, pseudofs, sizeof(pseudofs) / sizeof(pseudofs[0]),
+		sizeof(*pseudofs), typecmp) == NULL) {
+		return 0;
+	}
+
+	return 1;
+}
+
+int
+is_remotefs(const char *type)
+{
+	/* keep sorted for binary search */
+	static const char *remotefs[] = {
+		"afs",
+		"cifs",
+		"coda",
+		"ftpfs",
+		"fuse.sshfs",
+		"mfs",
+		"ncpfs",
+		"nfs",
+		"nfs4",
+		"smbfs",
+		"sshfs"
+	};
+
+	if (!type)
+		return 0;
+
+	if (bsearch(&type, remotefs, sizeof(remotefs) / sizeof(remotefs[0]),
+		sizeof(*remotefs), typecmp) == NULL) {
+		return 0;
+	}
+
+	return 1;
 }
